const helpers = require('../../helpers/helpers')
const mailService = require('pubsweet-component-mail-service')
const userHelper = require('../../helpers/User')

module.exports = models => async (req, res) => {
  const { collectionId, invitationId } = req.params
  const { invitationToken } = req.body

  if (!helpers.checkForUndefinedParams(invitationToken))
    return res.status(400).json({ error: 'Token is required' })

  try {
    const user = await models.User.findOneByField(
      'invitationToken',
      invitationToken,
    )
    const collection = await models.Collection.find(collectionId)
    const invitation = await collection.invitations.find(
      invitation => invitation.id === invitationId,
    )
    if (invitation === undefined)
      return res.status(404).json({
        error: `Invitation ${invitationId} not found`,
      })
    if (invitation.hasAnswer)
      return res
        .status(400)
        .json({ error: `Invitation has already been answered.` })
    if (invitation.userId !== user.id)
      return res.status(403).json({
        error: `User ${user.email} is not allowed to modify invitation ${
          invitation.id
        }`,
      })

    invitation.respondedOn = Date.now()
    invitation.hasAnswer = true
    invitation.isAccepted = false
    await collection.save()
    return await userHelper.setupReviewerDecisionEmailData({
      baseUrl: helpers.getBaseUrl(req),
      UserModel: models.User,
      FragmentModel: models.Fragment,
      collection,
      reviewerName: `${user.firstName} ${user.lastName}`,
      mailService,
      agree: false,
      user,
    })
  } catch (e) {
    const notFoundError = await helpers.handleNotFoundError(e, 'item')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
