import React from 'react'
import { get } from 'lodash'
import { Icon } from '@pubsweet/ui'
import { connect } from 'react-redux'
import styled from 'styled-components'
import { actions } from 'pubsweet-client'
import { compose, withState, withHandlers } from 'recompose'
import { getFormValues, reset as resetForm } from 'redux-form'

import { FormItems } from '../UIComponents'
import { StepOne, StepTwo, utils } from './'
import {
  selectError,
  selectFetching,
  createRecommendation,
} from '../../redux/recommendations'

const RecommendWizard = ({
  step,
  decision,
  nextStep,
  prevStep,
  closeModal,
  submitForm,
  isFetching,
  recommendationError,
  ...rest
}) => (
  <FormItems.RootContainer>
    <IconButton onClick={closeModal}>
      <Icon primary>x</Icon>
    </IconButton>
    {step === 0 && (
      <StepOne
        disabled={!decision}
        hideModal={closeModal}
        onSubmit={nextStep}
        {...rest}
      />
    )}
    {step === 1 && (
      <StepTwo
        decision={decision}
        goBack={prevStep}
        isFetching={isFetching}
        onSubmit={submitForm}
        recommendationError={recommendationError}
      />
    )}
  </FormItems.RootContainer>
)

export default compose(
  connect(
    state => ({
      isFetching: selectFetching(state),
      recommendationError: selectError(state),
      decision: get(getFormValues('recommendation')(state), 'decision'),
    }),
    {
      resetForm,
      createRecommendation,
      getCollections: actions.getCollections,
    },
  ),
  withState('step', 'changeStep', 0),
  withHandlers({
    nextStep: ({ changeStep }) => () => changeStep(s => s + 1),
    prevStep: ({ changeStep }) => () => changeStep(s => (s === 0 ? 0 : s - 1)),
    closeModal: ({ hideModal, resetForm }) => () => {
      hideModal()
      resetForm('recommendation')
    },
    submitForm: ({
      showModal,
      hideModal,
      resetForm,
      fragmentId,
      collectionId,
      getCollections,
      createRecommendation,
    }) => values => {
      const recommendation = utils.parseRecommendationValues(values)
      createRecommendation(collectionId, fragmentId, recommendation).then(r => {
        resetForm('recommendation')
        getCollections()
        showModal({
          title: 'Recommendation sent',
        })
      })
    },
  }),
)(RecommendWizard)

// #region styled components
const IconButton = styled.div`
  align-self: flex-end;
  cursor: pointer;
`

// #endregion
